/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.Association;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.realm.RegistryRealm;

public class AssociationsTest extends TestCase {

    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                EmbeddedRegistry embeddedRegistry = new InMemoryEmbeddedRegistry();
                registry = embeddedRegistry.getSystemRegistry();
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry. Caused by: " + e.getMessage());
        }
    }

    // todo: uncomment below part after completing the associations
    public void testSettingDependenciesInResources() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setContent("some content for first r1".getBytes());
        String path = registry.put("/depTest/test1/r1", r1);
        r1 = registry.get(path);

        Resource r2 = registry.newResource();
        r2.setContent("this is dependent on r1".getBytes());
        path = registry.put("/depTest/test1/r2", r2);
        r2 = registry.get(path);
        registry.addAssociation(r2.getPath(), "/depTest/test1/r1", Association.DEPENDENCY);
        registry.addAssociation(r1.getPath(), "/depTest/test1/r2", Association.DEPENDENCY);


        Resource r21 = registry.get("/depTest/test1/r1");
        Resource r22 = registry.get("/depTest/test1/r2");
        Association association[] = registry.getAllAssociations(r21.getPath());
        assertNotNull(association);
        boolean found = false;
        for (Association association1 : association) {
            if ("/depTest/test1/r2".equals(association1.getDestinationPath())) {
                found = true;
            }
        }
        assertTrue(found);
        association = registry.getAllAssociations(r22.getPath());
        assertNotNull(association);
        found = false;
        for (Association association1 : association) {
            if ("/depTest/test1/r1".equals(association1.getDestinationPath())) {
                found = true;
            }
        }
        assertTrue(found);
    }

    public void testSettingDependenciesFromAPI() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setContent("some content for second r1".getBytes());
        registry.put("/depTest/test2/r1", r1);

        Resource r2 = registry.newResource();
        r2.setContent("this is dependent on r2".getBytes());
        registry.put("/depTest/test2/r2", r2);

        registry.addAssociation("/depTest/test2/r2", "/depTest/test2/r1", Association.DEPENDENCY);
        registry.addAssociation("/depTest/test2/r1", "/depTest/test2/r2", Association.DEPENDENCY);

        Resource r21 = registry.get("/depTest/test2/r1");
        Resource r22 = registry.get("/depTest/test2/r2");
        Association association[] = registry.getAllAssociations(r21.getPath());
        assertNotNull(association);
        boolean found = false;
        for (Association association1 : association) {
            if ("/depTest/test2/r2".equals(association1.getDestinationPath())) {
                found = true;
            }
        }
        assertTrue(found);
        association = registry.getAllAssociations(r22.getPath());
        assertNotNull(association);
        found = false;
        for (Association association1 : association) {
            if ("/depTest/test2/r1".equals(association1.getDestinationPath())) {
                found = true;
            }
        }
        assertTrue(found);
    }

    public void testAssociationsAfterMove() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setContent("r1 content");
        registry.put("/test/move/a1/r1", r1);

        Resource r2 = registry.newResource();
        r2.setContent("r2 content");
        registry.put("/test/move/a1/r2", r2);

        registry.addAssociation("/test/move/a1/r1", "/test/move/a1/r2", "GENERAL");

        registry.move("/test/move/a1/r1", "/test/move/a2/r1");

        Association[] associations = registry.getAllAssociations("/test/move/a2/r1");
        assertEquals("Assocations are not moved correctly.", associations.length, 1);
        assertEquals("Assocations are not moved correctly.",
                associations[0].getSourcePath(), "/test/move/a2/r1");
        assertEquals("Assocations are not moved correctly.",
                associations[0].getDestinationPath(), "/test/move/a1/r2");
    }
}
